#include <stdio.h>
#include <stdlib.h>

#include <SDL/SDL.h>

#include "../nSX2.h"
#include "muGUI.h"

//KBF_Font *font;


int muGUI_Init ( muGUI * fgui ) {

	SDL_Surface  *video_screen;


   /* Create a 32-bit surface with the bytes of each pixel in R,G,B,A order,
       as expected by OpenGL for textures */
    Uint32 rmask, gmask, bmask, amask;

    /* SDL interprets each pixel as a 32-bit number, so our masks must depend
       on the endianness (byte order) of the machine */
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	    rmask = 0xff000000;
	    gmask = 0x00ff0000;
	    bmask = 0x0000ff00;
	    amask = 0x000000ff;
	#else
	    rmask = 0x000000ff;
	    gmask = 0x0000ff00;
	    bmask = 0x00ff0000;
	    amask = 0xff000000;
	#endif


	video_screen = SDL_GetVideoSurface ();

    fgui->MainScreen = SDL_CreateRGBSurface(SDL_SWSURFACE ,video_screen->w, video_screen->h, 32,rmask, gmask, bmask, amask);
    if(fgui->MainScreen == NULL) {
        printf("CreateRGBSurface failed: %s\n", SDL_GetError());
        return ( FALSE );
    }	
	
	fgui->Font1 = LoadFont("data/font_1.bmp");
	if (fgui->Font1 == NULL) {
		printf( "Couldnt load Font1\n");
		exit(1);
    }
	
	fgui->Font2 = LoadFont("data/font_2.bmp");
	if (fgui->Font2 == NULL) {
		printf( "Couldnt load Font2\n");
		exit(1);
    }
	
	
	
	fgui->nbWidget = 0;
	fgui->run = muGUI_MainLoop;
	
	fgui->clearScreen = muGUI_ClearScreen ;
	
	fgui->IdleProc = muGUI_defaultIdleProc;
	fgui->addWidget = muGUI_AddWidget;
	fgui->destroy = muGUI_Destroy;
	
	fgui->status = muGUI_STATUS_RUNNING;
	
	
	fgui->clearScreen(fgui);
	
	
	return ( TRUE );
}


void muGUI_Destroy (  void * this ) {
	

	muGUI * zGui;
	zGui = (muGUI*) this;

	printf("Destroy muGUI\n");
	
 	FreeFont(zGui->Font1);
    FreeFont(zGui->Font2);
	SDL_FreeSurface(zGui->MainScreen);

}



void muGUI_MainLoop (  void * this ) {
	
	unsigned int boucle;
	SDL_Surface * video_screen;
	int result;
   	SDL_Event event;
   	
   	int NeedGlobalRefresh;	
   	
  		
	muGUI * zGui;
	zGui = (muGUI*) this;

	video_screen = SDL_GetVideoSurface ();
	
	while ( zGui->status == muGUI_STATUS_RUNNING ) {
		
		
		
		zGui->IdleProc();

		
		NeedGlobalRefresh = FALSE;
	
		
		for ( boucle = 0 ; boucle < zGui->nbWidget ; boucle++) {
			
			if ( zGui->myWid[ boucle ].NeedRefresh == TRUE ) NeedGlobalRefresh = TRUE ;
		
		}
		
		if ( NeedGlobalRefresh == TRUE ) {
			zGui->clearScreen(zGui);
			for ( boucle = 0 ; boucle < zGui->nbWidget ; boucle++) {
				zGui->myWid[ boucle ].external_draw( &(zGui->myWid[ boucle ]) , zGui );
				
				if ( zGui->myWid[ boucle ].Status == MUGUI_WID_STATUS_SHOW ) 
					zGui->myWid[ boucle ].internal_draw( &(zGui->myWid[ boucle ]) , zGui );
				
				zGui->myWid[ boucle ].NeedRefresh = 0;
			}

			
						
			result = SDL_BlitSurface(zGui->MainScreen , NULL, video_screen, NULL);
			SDL_UpdateRect( video_screen,0,0,0,0 );
		
			
		}
		

		SDL_PollEvent(&event);

	}
}




void muGUI_AddWidget ( void * this , muGUIwidget * fWid ) {
	
	muGUI * zGui;
	zGui = (muGUI*) this;

	if ( fWid != NULL ) {

		zGui->myWid[ zGui->nbWidget ].ID = zGui->nbWidget ;
		zGui->myWid[ zGui->nbWidget ].Type = fWid->Type ;
		zGui->myWid[ zGui->nbWidget ].x = fWid->x ;
		zGui->myWid[ zGui->nbWidget ].y = fWid->y ;
		zGui->myWid[ zGui->nbWidget ].w = fWid->w ;
		zGui->myWid[ zGui->nbWidget ].h = fWid->h ;
		zGui->myWid[ zGui->nbWidget ].NeedRefresh = fWid->NeedRefresh ;
		zGui->myWid[ zGui->nbWidget ].Status = fWid->Status;
		zGui->myWid[ zGui->nbWidget ].external_draw = fWid->external_draw ;
		zGui->myWid[ zGui->nbWidget ].internal_draw = fWid->internal_draw ;
		zGui->myWid[ zGui->nbWidget ].isClick = fWid->isClick ;	
		
		
		zGui->myWid[ zGui->nbWidget ].image = fWid->image ;	
		zGui->myWid[ zGui->nbWidget ].addImage = fWid->addImage ;	
		
		zGui->myWid[ zGui->nbWidget ].TextBuffer = fWid->TextBuffer ;	
		zGui->myWid[ zGui->nbWidget ].addText  = fWid->addText;
		zGui->myWid[ zGui->nbWidget ].maxLines  = fWid->maxLines;
		zGui->myWid[ zGui->nbWidget ].displayAtLine = fWid->displayAtLine;
		
		
		strcpy( zGui->myWid[ zGui->nbWidget ].Title, fWid->Title );
		
		
		zGui->nbWidget = zGui->nbWidget + 1;
	}

	
	
}

void muGUI_ClearScreen (  void * this ) {
	
	SDL_Color rgb;
	Uint32 color;
 	
	muGUI * zGui;
	zGui = (muGUI*) this;



	
	rgb.r = 0x00;
	rgb.g = 0x69;	
	rgb.b = 0x99;	
	color = SDL_MapRGB(zGui->MainScreen->format, rgb.r, rgb.g, rgb.b);
 
   	SDL_FillRect(zGui->MainScreen, NULL, color);

	
}

void muGUI_defaultIdleProc ( void ) {
	//printf(".");
}

void PrintToEmuOutput (  char * ftext ) {
	
	nSX2GUI.myWid[nSX2_WID_EMU_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_EMU_OUTPUT],ftext);
	
	
}

void PrintToOpcodeOutput (  char * ftext ) {
	
	nSX2GUI.myWid[nSX2_WID_OPCODE_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_OPCODE_OUTPUT],ftext);
	
	
}

void PrintToRegistersOutput (  char * ftext ) {
	
	nSX2GUI.myWid[nSX2_WID_REGISTER_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_REGISTER_OUTPUT],ftext);
	
	
} 