/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <SDL/SDL.h>


#include "nSX2.h"
#include "plugin_gs_04/PS2Edefs.h"
#include "muGUI/muGUI.h"


Mips_CPU 		MipsCpu;
Bios_Hw  		Ps2Bios;

SDL_Surface		* 		MainScreen;
SDL_Event		*		MainSDLEvent;

muGUI nSX2GUI;


/* This function may run in a separate event thread */
int FilterEvents(const SDL_Event *event) {
    static int boycott = 1;
    static int DragAndDrop = 0;
	static int WidgetID_Click;
	int boucle;

	int result;

    /* This quit event signals the closing of the window */
    if ( (event->type == SDL_QUIT) && boycott ) {
        printf("Quit event filtered out -- try again.\n");
        boycott = 0;
        return(0);
    }
    
    if ( (event->type == SDL_QUIT) && !boycott ) {
        printf("Quit event filtered out -- try again.\n");
        boycott = 0;
        nSX2GUI.IdleProc=muGUI_nSX2EndingProc;
        return(0);
    }
    
    
    
    if ( event->type == SDL_MOUSEMOTION ) {
        	//printf("Mouse moved to (%d,%d)\n",event->motion.x, event->motion.y);
        
        if ( DragAndDrop == 1 ) {
        		nSX2GUI.myWid[ WidgetID_Click ].x = nSX2GUI.myWid[ WidgetID_Click ].x + event->motion.xrel ;
        		nSX2GUI.myWid[ WidgetID_Click ].y = nSX2GUI.myWid[ WidgetID_Click ].y + event->motion.yrel ;
        		
        		if ( nSX2GUI.myWid[ WidgetID_Click ].x  <= 0 ) nSX2GUI.myWid[ WidgetID_Click ].x = 0 ;
        		if ( nSX2GUI.myWid[ WidgetID_Click ].y  <= 0 ) nSX2GUI.myWid[ WidgetID_Click ].y = 0 ;
        		
        		nSX2GUI.myWid[ WidgetID_Click ].NeedRefresh = TRUE;
        	
        	}
        	
        	
        return(0);    /* Drop it, we've handled it */
    }
    
       if ( event->type == SDL_MOUSEBUTTONDOWN ) {
        	
        	
        	//Check TitleBarClick
        	WidgetID_Click = -1;
        	result = -1;
        	for ( boucle = 0 ; boucle < nSX2GUI.nbWidget ; boucle++) {
			
				result = nSX2GUI.myWid[ boucle ].isClick( &(nSX2GUI.myWid[ boucle ]) , &nSX2GUI ,event->button.x , event->button.y);
				
				switch ( result ) {
					
					case MUGUI_WID_TITLEBAR_OTHER: {
						DragAndDrop = 1 ;
						WidgetID_Click = boucle ;
						break;
					}
				
					case MUGUI_WID_TITLEBAR_BUTTON_1: {
						if ( nSX2GUI.myWid[ boucle ].Status == MUGUI_WID_STATUS_SHOW ) {
							 	nSX2GUI.myWid[ boucle ].previous_h = nSX2GUI.myWid[ boucle ].h ;
								nSX2GUI.myWid[ boucle ].h = 22;
								nSX2GUI.myWid[ boucle ].Status = MUGUI_WID_STATUS_HIDE;
							}
						else
							{
								nSX2GUI.myWid[ boucle ].h = nSX2GUI.myWid[ boucle ].previous_h;
								nSX2GUI.myWid[ boucle ].Status = MUGUI_WID_STATUS_SHOW;
							}
							
						
						
						break;
					}
				
				
				}
			
			}
        	
        	
        	
        return(0);    /* Drop it, we've handled it */
    }
    
    if ( event->type == SDL_MOUSEBUTTONUP ) {
        	
            if ( DragAndDrop == 1 ) {
            	DragAndDrop = 0 ;
            	WidgetID_Click = - 1;
        	
        	}

        	
        	
        	
        return(0);    /* Drop it, we've handled it */
    }
    
    
    
	if ( event->type ==  SDL_KEYDOWN ) {
		switch (event->key.keysym.sym)	{
	   		case SDLK_ESCAPE: nSX2GUI.IdleProc=muGUI_nSX2EndingProc;	;   break;
	   		case SDLK_F1:
	   		
	   			MipsCpu.halt = 1 ; 
	   			break;
	   		   		
	   	
	   		case SDLK_F2:    
	   			MipsCpu.halt = 0; 
	   			break;
	   	
   			case SDLK_F3: 
   				if ( debug_op == 0 ) debug_op = 1 ;
   				else debug_op = 0;
				break;

	   		default:
	    
	  	}
		return 0;
	}

    if ( event->type ==  SDL_KEYDOWN ) {
		switch (event->key.keysym.sym)
	  	{
	   		case SDLK_ESCAPE:     break;
	   		case SDLK_F1:     break;
   			case SDLK_F2:     break;
	   		case SDLK_F3:    break;
	   		default:
	    
	  	}
		return 0;
    }
    
    
    
    
    
    return(1);
}




char ElfToExecute[256];



void muGUI_nSX2InitProc ( void ) {
	char tempText[100];
	static int 	nsx2InitState = 0;
	static 	int	IsInitOK = 0;
	int 	result;				//fonction return 
	char 	GSMainTitle[256];	//main windows title	
	
	
	switch ( nsx2InitState ) {
	
		case 0:	{
				
			sprintf(tempText,"nSX2 %d.%d by Muad - 2002 ( build date : %s )",  nSX2_VERSION, nSX2_BUILD,sSX2_BUILD_DATE);
			PrintToEmuOutput(tempText);	
			
			sprintf(tempText,"|  web  : http://nsx2.emulation64.com");
			PrintToEmuOutput(tempText);	
			
			sprintf(tempText,"|_ mail : atreides.muaddib@caramail.com");
			PrintToEmuOutput(tempText);		
		
			IsInitOK = 0;

			break;
		}
		
		case 1: {
			/*Open the elf File */
			result = ElfOpenFile(ElfToExecute);					if ( result == FALSE )  IsInitOK++;
			break;			
		}
		
		case 2: {
			/*Open the BIOS File */
			result = BiosOpenFile( "bios/scph10000.bin" ) ;		if ( result == FALSE )  IsInitOK++;
			break;		
		}
		
		case 3: {
			/*Various Memory Allocation */
			result = MipsMemoryAlloc();							if ( result == FALSE )  IsInitOK++;
			result = MipsSpecialAreaMemoryAlloc( );				if ( result == FALSE )  IsInitOK++;
			result = MipsGSAreaMemoryAlloc();					if ( result == FALSE )  IsInitOK++;
			result = MipsBiosMemoryAlloc( );					if ( result == FALSE )  IsInitOK++;
			break;
		}
	

		case 4: {
			/*If something wrong*/
			if ( IsInitOK != 0 ) {
				sprintf(tempText,"FATAL ERROR while emulator initialisation....Sorry Exiting");	
				PrintToEmuOutput(tempText);
				nSX2GUI.status = muGUI_STATUS_STOP;
			}
			break;
		}
	
		case 5: {
			/*Read the header of the Elf File*/
			ElfLoadHeader();
		
			/* */
			//ElfPrintInfo();
			//ElfPrintSectionInfo( 7 ) ;
			//getchar();
			break;
		}
	
		case 6: {	

			/*Copy the program to memory according the elf header*/
			CopyProgramToMemory(  ) ;
			break;
		}

		case 7: {	

			/*Copy data to memory according the elf header*/
			CopySectionToMemory( );
			break;
		}
	
		case 8: {
				
			/*Copy Bios to Memory */
			CopyBiosToMemory ( );	
			break;	
		}
	
	
		case 9: {	
			/* Initialisation of GS plugin */
			GSinit();
			result = GSopen(MainScreen ,GSMainTitle );
		
			if ( result != 0 ) {
				sprintf(tempText,"FATAL ERROR while graphics initialisation...Sorry Exiting");
				PrintToEmuOutput(tempText);
				nSX2GUI.status = muGUI_STATUS_STOP;
				
			} else {
				sprintf(tempText,"GS:OK");
				PrintToEmuOutput(tempText);			
			}
			break;	
		}


		case 10: {	
			/*Various PS2 Initialisation */
			Ps2Bios.NeedUpdateScreen 			= 0x6FFF;
			MipsCpu.PC 							= MipsCpu.EntryPointAdd ;	//Start Program At Addr
			MipsCpu.GPR[CPU_REG_GP].lo			= 0x81F80000;				//GP
			MipsCpu.GPR[CPU_REG_SP].lo			= 0x81F00000;				//SP
			MipsCpu.CPR0[12][0]					= 0x10900001;				//Reg Status of COP0
			MipsCpu.CPR0[15][0]					= 0x2E20	;				//Reg PrID of COP0
			MipsCpu.Nb_instructions 			= 0;
			Ps2Bios.NbSema 						= 0;
			
			
			/*
			Create some pointeurs to val for performing the conversion by value of :
			int to float 
			and 
			float to int
			without change the way of store
			for exemple :
				0x4863D000 = 233280.0000
				and not
				0x4863D000 = 1214500864.0000
			*/
			for ( result = 0 ; result < 32 ; result++ ) {
				MipsCpu.FPR1_p_float[result] = (float * )&(MipsCpu.FPR1[result].lo);	//Create a pointer
				MipsCpu.FPR1_p_u64[result] = (_u64 * )&(MipsCpu.FPR1_float[result]);	//Create a pointer
			}
			break;
		}	


		case 11: {
			/*Change the Idle proc of the Gui*/
			nSX2GUI.IdleProc=muGUI_nSX2Proc;
			MipsCpu.halt = 0;
			//strcpy(nSX2GUI.myWid[ nSX2_WID_EMU_OUTPUT ].Title , "EmuOutput - Running");
			
			/*Start Here */
			sprintf(tempText,"Start Interpreter....");
			PrintToEmuOutput(tempText);	
			break;
		}
		
		default: {
			sprintf(tempText,"Too Bad Muad...");
			PrintToEmuOutput(tempText);	
			break;
		}
		
	}
	


	
	nsx2InitState ++;
	
}


void muGUI_nSX2Proc ( void ) {
	
		// For debugging... 
		//if ( MipsCpu.PC == 0x00400e20) debug_op = 1;				//...after acessing a opcode
		//if ( MipsCpu.Nb_instructions > 557000 ) debug_op = 1;		//...after a number of opcode
		
		//if ( MipsCpu.PC == 0x00202930 ) result++;	
		//if ( result == 1 ) debug_op = 1;	
				
				
				
	if ( MipsCpu.halt == 0 ) {
		//Execute the current opcode
		MipsUpdateCpu();

		//Execute DMA stuff
		DmaUpdate();
		
		//Execute GS stuff
		GSUpdate();
		
	}	
}



void muGUI_nSX2EndingProc ( void ) {

	static int 	nsx2EndingState = 0;
	char tempText[100];
	
		
	switch ( nsx2EndingState ) {
		case 0:	{
			sprintf(tempText,"...Stop Interpreter");	
			PrintToEmuOutput(tempText);	
			break;
		}
			
		case 1:	{		
			sprintf(tempText,"Stop at PC = 0x%8.8llx after %lld instructions \n",MipsCpu.PC, MipsCpu.Nb_instructions);
			PrintToEmuOutput(tempText);	
			/*
			PrintDebugOpcode	();
			MipsCpuPrintReg		();
			MipsCpuFPR1PrintReg	();
			*/
			break;
		}
		
		case 2:	{
			/* Ending */
			ElfCloseFile();
			BiosCloseFile( ) ;
			break;
		}
		
		case 3:	{	
			/*Free All Memory*/
			FreeAllMemory();
			break;
		}
		
		case 4:	{	
			/*Free GS plugin Memory*/
			GSclose();
			break;
		}
		
		case 5:	{
			SDL_Delay(2000);
			break;
		}
		
		
		case 6:	{	
			nSX2GUI.status = muGUI_STATUS_STOP;
			break;	
		}
		
		
		
	}
		
	
	nsx2EndingState++;
}
 




int main( void ) {
	
	int 	result;				//fonction return 
	int		IsInitOK;			//general initialisation test
	char 	GSMainTitle[256];	//main windows title
	
	SDL_Surface *screen;				//The Video Screen
	
    Uint32 rmask, gmask, bmask, amask;	//Mask for the SDL Surface
	muGUIwidget tempWidget;				//Temporary Widget

	/* Initialize SDL */
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);


	/* Read config File */
	result = nSX2LoadConfig();							if ( result == FALSE )  IsInitOK++;


	/* Get a video mode for display */
	screen = SDL_SetVideoMode(800, 600, muGUI_MainScreenBPP, SDL_HWSURFACE );
	if ( screen == NULL ) {
		fprintf(stderr, "Couldn't set video mode: %s\n",SDL_GetError());
		exit(1);
	}
	
	/* SDL Filter Event for the GUI*/
   	SDL_SetEventFilter(FilterEvents);


	/*
	WARNING: MASK NOT USABLE FOR AN ANOTHER APP
	*/

	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	    rmask = 0x0000ff00;
	    gmask = 0x00ff0000;
	    bmask = 0xff000000;
	    amask = 0x000000ff;
	#else
	    rmask = 0x00ff0000;
	    gmask = 0x0000ff00;
	    bmask = 0x000000ff;
	    amask = 0xff000000;
	#endif
	



	/*Creation of the MainPS2Output Video*/
	MainScreen = SDL_CreateRGBSurface( SDL_HWSURFACE  , muGUI_MainScreenW,muGUI_MainScreenH, muGUI_MainScreenBPP,rmask, gmask, bmask, amask);
    if(MainScreen == NULL) {
        printf("CreateRGBSurface failed: %s\n", SDL_GetError());
        IsInitOK++;
    }	

	/**************************************************************************************************/


	/*GUI Initialisation */
	muGUI_Init ( &nSX2GUI );
	
	/*Add the main PS2 screen to the GUI*/
	muGUIwidget_init ( &tempWidget , MUGUI_WID_IMAGE , 10 ,10 , muGUI_MainScreenW+2 , muGUI_MainScreenH+22 , "VideoOutput" );
	tempWidget.addImage(&tempWidget,MainScreen);
	nSX2GUI.addWidget(&nSX2GUI,&tempWidget);
	
	
	/*Add the EmuOutput widget*/
	muGUIwidget_init ( &tempWidget , MUGUI_WID_TEXT , 10 , muGUI_MainScreenH+42 , 500 , 170 , "EmuOutput" );
	nSX2GUI.addWidget(&nSX2GUI,&tempWidget);

	//Add the OpcodeOutput widget
	muGUIwidget_init ( &tempWidget , MUGUI_WID_TEXT , 500 + 20 , muGUI_MainScreenH+42 , 250 , 170 , "Opcodes" );
	nSX2GUI.addWidget(&nSX2GUI,&tempWidget);

	//Add the RegisterOutput widget
	muGUIwidget_init ( &tempWidget , MUGUI_WID_TEXT , 20 + 500 , 10 , 180 , 378 + 22 , "CpuRegisters" );
	nSX2GUI.addWidget(&nSX2GUI,&tempWidget);
		
	/*Change the Windows title*/
	sprintf(GSMainTitle, " %s %d.%2.2d - %s", "nSX2", nSX2_VERSION, nSX2_BUILD,ElfToExecute);
	SDL_WM_SetCaption ( GSMainTitle , NULL );
	

	/*Init Start Here*/		
	nSX2GUI.IdleProc=muGUI_nSX2InitProc;
	nSX2GUI.run(&nSX2GUI);	
	
	/*Destroy all Gui elements*/
	nSX2GUI.destroy(&nSX2GUI);
	
	/*clean Quit SDL*/
	SDL_Quit();

	/*All done...*/	
	return TRUE;
	
}
